<?php































require_once(INCLUDE_PATH."/common.php");

class DB_pgsql extends DB_common
{
 

 var $connection;
 var $phptype, $dbsyntax;
 var $prepare_tokens = array();
 var $prepare_types = array();
 var $transaction_opcount = 0;
 var $dsn = array();
 var $row = array();
 var $num_rows = array();
 var $affected = 0;
 var $autocommit = true;
 var $fetchmode = DB_FETCHMODE_ORDERED;

 
 

 function DB_pgsql()
 {
 $this->DB_common();
 $this->phptype = 'pgsql';
 $this->dbsyntax = 'pgsql';
 $this->features = array(
 'prepare' => false,
 'pconnect' => true,
 'transactions' => true,
 'limit' => 'alter'
 );
 $this->errorcode_map = array(
 );
 }

 
 

 








 function connect($dsninfo, $persistent = false)
 {
 if (!DB::assertExtension('pgsql'))
 return $this->raiseError(DB_ERROR_EXTENSION_NOT_FOUND);

 $this->dsn = $dsninfo;
 $protocol = (isset($dsninfo['protocol'])) ? $dsninfo['protocol'] : 'tcp';
 $connstr = '';

 if ($protocol == 'tcp') {
 $connstr = 'host=' . $dsninfo['hostspec'];
 if (!empty($dsninfo['port'])) {
 $connstr .= ' port=' . $dsninfo['port'];
 }
 }

 if (isset($dsninfo['database'])) {
 $connstr .= ' dbname=' . $dsninfo['database'];
 }
 if (!empty($dsninfo['username'])) {
 $connstr .= ' user=' . $dsninfo['username'];
 }
 if (!empty($dsninfo['password'])) {
 $connstr .= ' password=' . $dsninfo['password'];
 }
 if (!empty($dsninfo['options'])) {
 $connstr .= ' options=' . $dsninfo['options'];
 }
 if (!empty($dsninfo['tty'])) {
 $connstr .= ' tty=' . $dsninfo['tty'];
 }

 $connect_function = $persistent ? 'pg_pconnect' : 'pg_connect';
 
 ob_start();
 $conn = $connect_function($connstr);
 $error = ob_get_contents();
 ob_end_clean();
 if ($conn == false) {
 return $this->raiseError(DB_ERROR_CONNECT_FAILED, null,
 null, null, strip_tags($error));
 }
 $this->connection = $conn;
 return DB_OK;
 }

 
 

 




 function disconnect()
 {
 $ret = @pg_close($this->connection); 
 $this->connection = null;
 return $ret;
 }

 
 

 









 function simpleQuery($query)
 {
 $ismanip = DB::isManip($query);
 $this->last_query = $query;
 $query = $this->modifyQuery($query);
 if (!$this->autocommit && $ismanip) {
 if ($this->transaction_opcount == 0) {
 $result = @pg_exec($this->connection, "begin;");
 if (!$result) {
 return $this->pgsqlRaiseError();
 }
 }
 $this->transaction_opcount++;
 }
 $result = @pg_exec($this->connection, $query);
 if (!$result) {
 return $this->pgsqlRaiseError();
 }
 
 
 if ($ismanip) {
 $this->affected = @pg_cmdtuples($result);
 return DB_OK;
 } elseif (preg_match('/^\s*\(?\s*SELECT\s+/si', $query) &&
 !preg_match('/^\s*\(?\s*SELECT\s+INTO\s/si', $query)) {
 






 $this->row[$result] = 0; 
 $numrows = $this->numrows($result);
 if (is_object($numrows)) {
 return $numrows;
 }
 $this->num_rows[$result] = $numrows;
 $this->affected = 0;
 return $result;
 } else {
 $this->affected = 0;
 return DB_OK;
 }
 }

 
 

 








 function nextResult($result)
 {
 return false;
 }

 
 

 











 function errorCode($errormsg)
 {
 static $error_regexps;
 if (empty($error_regexps)) {
 $error_regexps = array(
 '/(Table does not exist\.|Relation [\"\'].*[\"\'] does not exist|sequence does not exist|class ".+" not found)$/' => DB_ERROR_NOSUCHTABLE,
 '/Relation [\"\'].*[\"\'] already exists|Cannot insert a duplicate key into (a )?unique index.*/' => DB_ERROR_ALREADY_EXISTS,
 '/divide by zero$/' => DB_ERROR_DIVZERO,
 '/pg_atoi: error in .*: can\'t parse /' => DB_ERROR_INVALID_NUMBER,
 '/ttribute [\"\'].*[\"\'] not found$|Relation [\"\'].*[\"\'] does not have attribute [\"\'].*[\"\']/' => DB_ERROR_NOSUCHFIELD,
 '/parser: parse error at or near \"/' => DB_ERROR_SYNTAX,
 '/referential integrity violation/' => DB_ERROR_CONSTRAINT
 );
 }
 foreach ($error_regexps as $regexp => $code) {
 if (preg_match($regexp, $errormsg)) {
 return $code;
 }
 }
 
 return DB_ERROR;
 }

 
 







 function fetchRow($result, $fetchmode = DB_FETCHMODE_DEFAULT, $rownum=null)
 {
 if ($fetchmode == DB_FETCHMODE_DEFAULT) {
 $fetchmode = $this->fetchmode;
 }
 $res = $this->fetchInto ($result, $arr, $fetchmode, $rownum);
 if ($res !== DB_OK) {
 return $res;
 }
 return $arr;
 }

 

 









 function fetchInto($result, &$row, $fetchmode, $rownum=null)
 {
 $rownum = ($rownum !== null) ? $rownum : $this->row[$result];
 if ($rownum >= $this->num_rows[$result]) {
 return null;
 }
 if ($fetchmode & DB_FETCHMODE_ASSOC) {
 $row = @pg_fetch_array($result, $rownum, PGSQL_ASSOC);
 } else {
 $row = @pg_fetch_row($result, $rownum);
 }
 if (!$row) {
 $err = pg_errormessage($this->connection);
 if (!$err) {
 return null;
 }
 return $this->pgsqlRaiseError();
 }
 $this->row[$result] = ++$rownum;
 return DB_OK;
 }

 
 

 






 function freeResult($result)
 {
 if (is_resource($result)) {
 return @pg_freeresult($result);
 }
 if (!isset($this->prepare_tokens[(int)$result])) {
 return false;
 }
 unset($this->prepare_tokens[(int)$result]);
 unset($this->prepare_types[(int)$result]);
 unset($this->row[(int)$result]);
 unset($this->num_rows[(int)$result]);
 $this->affected = 0;
 return true;
 }

 
 
 





 function quote($str = null)
 {
 switch (strtolower(gettype($str))) {
 case 'null':
 return 'NULL';
 case 'integer':
 case 'double' :
 return $str;
 case 'string':
 default:
 $str = str_replace("'", "''", $str);
 
 $str = str_replace('\\', '\\\\', $str);
 return "'$str'";
 }
 }
 
 

 






 function numCols($result)
 {
 $cols = @pg_numfields($result);
 if (!$cols) {
 return $this->pgsqlRaiseError();
 }
 return $cols;
 }

 
 

 






 function numRows($result)
 {
 $rows = @pg_numrows($result);
 if ($rows === null) {
 return $this->pgsqlRaiseError();
 }
 return $rows;
 }

 
 

 





 function errorNative()
 {
 return pg_errormessage($this->connection);
 }

 
 

 


 function autoCommit($onoff = false)
 {
 
 
 $this->autocommit = $onoff ? true : false;
 return DB_OK;
 }

 
 

 


 function commit()
 {
 if ($this->transaction_opcount > 0) {
 
 
 $result = @pg_exec($this->connection, "end;");
 $this->transaction_opcount = 0;
 if (!$result) {
 return $this->pgsqlRaiseError();
 }
 }
 return DB_OK;
 }

 
 

 


 function rollback()
 {
 if ($this->transaction_opcount > 0) {
 $result = @pg_exec($this->connection, "abort;");
 $this->transaction_opcount = 0;
 if (!$result) {
 return $this->pgsqlRaiseError();
 }
 }
 return DB_OK;
 }

 
 

 





 function affectedRows()
 {
 return $this->affected;
 }
 
 

 










 function nextId($seq_name, $ondemand = true)
 {
 $seqname = $this->getSequenceName($seq_name);
 $repeat = 0;
 do {
 $this->pushErrorHandling(PEAR_ERROR_RETURN);
 $result = $this->query("SELECT NEXTVAL('${seqname}')");
 $this->popErrorHandling();
 if ($ondemand && DB::isError($result) &&
 $result->getCode() == DB_ERROR_NOSUCHTABLE) {
 $repeat = 1;
 $result = $this->createSequence($seq_name);
 if (DB::isError($result)) {
 return $this->raiseError($result);
 }
 } else {
 $repeat = 0;
 }
 } while ($repeat);
 if (DB::isError($result)) {
 return $this->raiseError($result);
 }
 $arr = $result->fetchRow(DB_FETCHMODE_ORDERED);
 $result->free();
 return $arr[0];
 }

 
 

 






 function createSequence($seq_name)
 {
 $seqname = $this->getSequenceName($seq_name);
 $this->pushErrorHandling(PEAR_ERROR_RETURN);
 $result = $this->query("CREATE SEQUENCE ${seqname}");
 $this->popErrorHandling();
 return $result;
 }

 
 

 






 function dropSequence($seq_name)
 {
 $seqname = $this->getSequenceName($seq_name);
 return $this->query("DROP SEQUENCE ${seqname}");
 }

 
 

 function modifyLimitQuery($query, $from, $count)
 {
 $query = $query . " LIMIT $count, $from";
 return $query;
 }

 
 

 function pgsqlRaiseError($errno = null)
 {
 $native = $this->errorNative();
 if ($errno === null) {
 $err = $this->errorCode($native);
 } else {
 $err = $errno;
 }
 return $this->raiseError($err, null, null, null, $native);
 }

 
 

 









 function _pgFieldFlags($resource, $num_field, $table_name)
 {
 $field_name = @pg_fieldname($resource, $num_field);

 $result = pg_exec($this->connection, "SELECT f.attnotnull, f.atthasdef
                                FROM pg_attribute f, pg_class tab, pg_type typ
                                WHERE tab.relname = typ.typname
                                AND typ.typrelid = f.attrelid
                                AND f.attname = '$field_name'
                                AND tab.relname = '$table_name'");
 if (pg_numrows($result) > 0) {
 $row = pg_fetch_row($result, 0);
 $flags = ($row[0] == 't') ? 'not_null ' : '';

 if ($row[1] == 't') {
 $result = pg_exec($this->connection, "SELECT a.adsrc
                                    FROM pg_attribute f, pg_class tab, pg_type typ, pg_attrdef a
                                    WHERE tab.relname = typ.typname AND typ.typrelid = f.attrelid
                                    AND f.attrelid = a.adrelid AND f.attname = '$field_name'
                                    AND tab.relname = '$table_name'");
 $row = pg_fetch_row($result, 0);
 $num = str_replace('\'', '', $row[0]);

 $flags .= "default_$num ";
 }
 }
 $result = pg_exec($this->connection, "SELECT i.indisunique, i.indisprimary, i.indkey
                                FROM pg_attribute f, pg_class tab, pg_type typ, pg_index i
                                WHERE tab.relname = typ.typname
                                AND typ.typrelid = f.attrelid
                                AND f.attrelid = i.indrelid
                                AND f.attname = '$field_name'
                                AND tab.relname = '$table_name'");
 $count = pg_numrows($result);

 for ($i = 0; $i < $count ; $i++) {
 $row = pg_fetch_row($result, $i);
 $keys = explode(" ", $row[2]);

 if (in_array($num_field + 1, $keys)) {
 $flags .= ($row[0] == 't') ? 'unique ' : '';
 $flags .= ($row[1] == 't') ? 'primary ' : '';
 if (count($keys) > 1)
 $flags .= 'multiple_key ';
 }
 }

 return trim($flags);
 }

 
 

 










 function tableInfo($result, $mode = null)
 {
 $count = 0;
 $id = 0;
 $res = array();

 




































 
 

 if (is_string($result)) {
 $id = pg_exec($this->connection,"SELECT * FROM $result");
 if (empty($id)) {
 return $this->pgsqlRaiseError();
 }
 } else { 
 $id = $result;
 if (empty($id)) {
 return $this->pgsqlRaiseError();
 }
 }

 $count = @pg_numfields($id);

 
 if (empty($mode)) {

 for ($i=0; $i<$count; $i++) {
 $res[$i]['table'] = (is_string($result)) ? $result : '';
 $res[$i]['name'] = @pg_fieldname ($id, $i);
 $res[$i]['type'] = @pg_fieldtype ($id, $i);
 $res[$i]['len'] = @pg_fieldsize ($id, $i);
 $res[$i]['flags'] = (is_string($result)) ? $this->_pgFieldflags($id, $i, $result) : '';
 }

 } else { 
 $res["num_fields"]= $count;

 for ($i=0; $i<$count; $i++) {
 $res[$i]['table'] = (is_string($result)) ? $result : '';
 $res[$i]['name'] = @pg_fieldname ($id, $i);
 $res[$i]['type'] = @pg_fieldtype ($id, $i);
 $res[$i]['len'] = @pg_fieldsize ($id, $i);
 $res[$i]['flags'] = (is_string($result)) ? $this->_pgFieldFlags($id, $i, $result) : '';
 if ($mode & DB_TABLEINFO_ORDER) {
 $res['order'][$res[$i]['name']] = $i;
 }
 if ($mode & DB_TABLEINFO_ORDERTABLE) {
 $res['ordertable'][$res[$i]['table']][$res[$i]['name']] = $i;
 }
 }
 }

 
 if (is_resource($id)) {
 @pg_freeresult($id);
 }
 return $res;
 }

 
 

 




 function getSpecialQuery($type)
 {
 switch ($type) {
 case 'tables': {
 $sql = "SELECT c.relname as \"Name\"
                        FROM pg_class c, pg_user u
                        WHERE c.relowner = u.usesysid AND c.relkind = 'r'
                        AND not exists (select 1 from pg_views where viewname = c.relname)
                        AND c.relname !~ '^pg_'
                        UNION
                        SELECT c.relname as \"Name\"
                        FROM pg_class c
                        WHERE c.relkind = 'r'
                        AND not exists (select 1 from pg_views where viewname = c.relname)
                        AND not exists (select 1 from pg_user where usesysid = c.relowner)
                        AND c.relname !~ '^pg_'";
 break;
 }
 case 'views': {
 
 $sql = "SELECT viewname FROM pg_views";
 break;
 }
 case 'users': {
 
 $sql = 'SELECT usename FROM pg_user';
 break;
 }
 case 'databases': {
 $sql = 'SELECT datname FROM pg_database';
 break;
 }
 case 'functions': {
 $sql = 'SELECT proname FROM pg_proc';
 break;
 }
 default:
 return null;
 }
 return $sql;
 }

 

}





?>
